<?php

namespace app\Types\DatabaseObjects;

use app\Exceptions\AccountNotFoundException;
use app\Interfaces\IDatabaseObject;
use app\Hajeebtok;
use app\Logger;
use Hajeebtok\Types\Exceptions\SecurityFaultException;

class Account implements IDatabaseObject
{

	public private(set) ?int $id;
	public private(set) ?string $username;
	public private(set) ?string $password;
	public private(set) ?string $picture_hash;
	public private(set) ?bool $verified;
	public private(set) ?string $bio;

	public function __construct(?int $id = null, ?string $username = null, ?string $password=null, ?string $picture_hash=null, ?bool $verified=null, ?string $bio=null)
	{
		$this->id = $id;
		$this->username = $username;
		$this->password = $password ? password_hash($password, PASSWORD_DEFAULT) : null;
		$this->picture_hash = $picture_hash;
		$this->verified = $verified;
		$this->bio = $bio;
	}

	/**
	 * Creates the table for the object type in the database.
	 */
	public static function CreateTable(): void
	{
		throw new SecurityFaultException("Attempt to create table on user object.");
	}

	/**
	 * Drops the table for the object type from the database.
	 */
	public static function DropTable(): void
	{
		throw new SecurityFaultException("Attempt to drop table on user object.");
	}
	
	/**
	 * Saves the object to the database.
	 */
	public function Save(): int
	{
		Hajeebtok::$Database->Query("INSERT INTO accounts (username, password, picture_hash, verified, bio) VALUES (:username, :password, :picture_hash, :verified, :bio)", [
			"username" => $this->username,
			"password" => $this->password,
			"picture_hash" => $this->picture_hash,
			"verified" => $this->verified,
			"bio" => $this->bio
		]);
		$id = Hajeebtok::$Database->LastInsertId();
		Logger::Debug("Saved account id ($id).");
        return $id;
	}

	/**
	 * Deletes the object from the database.
	 */
	public function Delete()
	{
		Hajeebtok::$Database->Query("DELETE FROM accounts WHERE id = :id", ["id" => $this->id]);
	}

	/**
	 * Loads the object from the database.
	 */
	public function Load()
	{
		if (empty($this->id)) { // search by username
            $data = Hajeebtok::$Database->Row("SELECT * FROM accounts WHERE username = :username", ["username" => $this->username]);
        } else {
            // search by id
            $data = Hajeebtok::$Database->Row("SELECT * FROM accounts WHERE id = :id", ["id" => $this->id]);
        }

		if (empty($data)) throw new AccountNotFoundException($this->id, 404);

        $this->id = $data["id"];
		$this->username = $data["username"];
		$this->password = $data["password"];
		$this->picture_hash = $data["picture_hash"];
		$this->verified = $data["verified"];
		$this->bio = $data["bio"];
	}

    public function Update() {
        Hajeebtok::$Database->Query("UPDATE accounts SET username = :username, password = :password, picture_hash = :picture_hash, verified = :verified, bio = :bio WHERE id = :id", [
            "id" => $this->id,
            "username" => $this->username,
            "password" => $this->password,
            "picture_hash" => $this->picture_hash,
            "verified" => $this->verified,
            "bio" => $this->bio
        ]);
        Logger::Debug("Updated account id ($this->id).");;
    }
    
    public function LoadMany(): array {
        if(empty($this->username)) throw new AccountNotFoundException(0, 404);
        $data = Hajeebtok::$Database->Query("SELECT * FROM accounts WHERE username LIKE :username", ["username" => "%$this->username%"]);
        if(empty($data)) throw new AccountNotFoundException(0, 404);
        return $data;
    }

    public function Exists(): bool {
        if ($this->id === null) { // search by username
            $data = Hajeebtok::$Database->Row("SELECT * FROM accounts WHERE username = :username", ["username" => $this->username]);
        } else { // search by id
            $data = Hajeebtok::$Database->Row("SELECT * FROM accounts WHERE id = :id", ["id" => $this->id]);
        }
        return !empty($data);
    }
}
